/**
 * Reset User Password script
 *
 * @method POST
 * @param json {string}
 *    {
 *       email: "email"
 *    }
 */
model.result = false;
model.message = "";
var s = new XML(config.script);

function getRandomNum(lbound, ubound)
{
   return (Math.floor(Math.random() * (ubound - lbound)) + lbound);
}
function getRandomChar()
{
   var chars = s["pw-chars"].toString();
   return chars.charAt(getRandomNum(0, chars.length));
}
function getRandomPassword(n)
{
   var password = "";
   for (var i=0; i<n; i++)
   {
      password += getRandomChar();
   }
   return password;
}
function resetPassword(u)
{
   var pwlen = parseInt(s["pw-length"].toString(), 10);
   // Auto-generate a password if not specified
   if (u.password == null || u.password == "")
   {
      u.password = getRandomPassword(pwlen);
   }
   lysePerson.setPasswordWithTenant(u.userName, u.password, u.tenant);
   return u;
}
function mailResetPasswordNotification(u,message,shareUrl)
{
   // create mail action
   var fromName = person.properties.firstName + " " + person.properties.lastName;

   // Format email message, this message will contain all accounts.
   var message = msg.get("template.passwordReset", [u.firstName, u.lastName, encodeURIComponent(u.userName), message, shareUrl, fromName]);

   lysePerson.sendMail("mail", u.email, getAdminsEmail(), msg.get("subject.text"), message);
}
function logResetPasswordResults(users)
{
   var logContent = "";
   for (var i=0; i<users.length; i++)
   {
      logContent += (users[i].userName + "," + users[i].password + "\n");
   }
   var d = new Date();
   var logFile = userhome.createFile("reset_password_" + d.getTime() + ".csv");
   logFile.content = logContent;
   logFile.save();
   return logFile;
}
function getUserByUserName(username)
{
   return people.getPerson(username);
}

function getAdminsEmail()
{
    var admin = people.getPerson('admin');
    return admin.properties.email;
}
function userIsMember(u, g)
{
   var members = people.getMembers(g);
   for (var i=0; i<members.length; i++)
   {
      if (members[i].properties.userName == u.properties.userName)
      {
         return true;
      }
   }
   return false;
}
function main()
{
   var user, u, email, shareUrl, users,
       logResults = s["log-resets"].toString() == "true",
       disallowedUsers = s["disallowed-users"].toString().split(",");

   if ((json.isNull("email")) || (json.get("email") == null) || (json.get("email").length() == 0))
   {
      status.setCode(status.STATUS_BAD_REQUEST, msg.get("error.noEmail"));
      status.redirect = true;
      return;
   }

   email = json.get("email");
   users = lysePerson.getUsers(email);
   shareUrl = scriptHelper.getShareUrl();

   if(users === undefined)
   {
      status.setCode(status.STATUS_OK, msg.get("error.notFound", [email]));
      status.redirect = true;
      return;
   }

   if (users.length == 0)
   {
      status.setCode(status.STATUS_OK, msg.get("error.notFound", [email]));
      status.redirect = true;
      return;
   }

   var userCredentials = "<br>";
   var lastUser;
   //Added for-loop multiple users due issue: #45141
   for (var j = 0; j < users.size(); j++)
   {
	   user = users.get(j);

	   if (!lysePerson.isEnabled(user.userName, user.tenant))
	   {
   		  //Discard this if multiple users but skip item as its disabled due: #45141,
   		  if(users.length == 1){
		      status.setCode(status.STATUS_FORBIDDEN, msg.get("error.disabled"));
		      status.redirect = true;
		      return;
	      }else{
	      	continue;
	      }
	   }

	   for ( var i = 0; i < disallowedUsers.length; i++ )
	   {
	      if (user.userName == disallowedUsers[i])
	      {
	   		  //Discard this if multiple users but skip item as its disabled due: #45141,
	   		  if(users.length == 1){
		         status.setCode(status.STATUS_FORBIDDEN, msg.get("error.disallowed"));
		         status.redirect = true;
		         return;
		      }else{
	      		continue;
	      	  }
	      }
	   }

	   // Reset the password
	   try
	   {
	      //Try local password
		   u = resetPassword(user);
		   lastUser = u;

		   // Format and add string of each account to a single string.
		   userCredentials += msg.get("template.account", [u.userName, u.password]);
	   }
	   catch (e)
	   {
         logger.error(e);
         status.setCode(status.STATUS_BAD_REQUEST, msg.get("error.notLocal"));
         status.redirect = true;
         return;
	   }

	   if (logResults)
	   {
	      model.resultsLog = logResetPasswordResults([u]);
	   }
   }

   // Send e-mail confirmation
   try
   {
	  mailResetPasswordNotification(lastUser,userCredentials,shareUrl);
   }
   catch (e)
   {
      status.setCode(status.STATUS_INTERNAL_SERVER_ERROR, msg.get("error.mail"));
      status.redirect = true;
      return;
   }
   model.success = true;
}

main();
